// =========================================================================================================
//  Object Placer
//  Version: 1.3.0
//  Author: Kronzky (www.kronzky.info / kronzky@gmail.com)
// ---------------------------------------------------------------------------------------------------------
//
// Fills a marker-defined area with objects
//
// There are two ways to use this script. Either to create a regular matrix with predefined
// distances between the objects, or to create a specified number of objects at random positions 
// within the marker area.
//
// To create a MATRIX of objects, use this syntax:
//    nil=["markername", "objectname", height, xdist, ydist] execVM "objectplacer.sqf"
//
//    Parameters:
//      "markername" : name of marker that designates the area to be covered
//      "objectname" : class name of the objects to be created 
//                     (check the Wiki for class names: http://community.bistudio.com/wiki/Category:ArmA_Classes)
//      height       : the height the objects should be created at 
//      xdist, ydist : distance in x and y the created objects should have 
//
//  e.g.: nil=["paraiso", "BO_GBU12_LGB", 50, 10, 10] execVM "objectplacer.sqf" 
//           will create bombs at a height of 50, spaced 10 meters apart, over a marker called "paraiso"
//
// ---------------------------------------------------------------------------------------------------------
//
// To create a objects at RANDOM positions, use this syntax:
//    nil=["markername", "objectname", height, amount, delay, "random"] execVM "objectplacer.sqf"
//
//    Parameters:
//      "markername" : name of marker that designates the area to be covered
//      "objectname" : class name of the objects to be created 
//                     (check the Wiki for class names: http://community.bistudio.com/wiki/Category:ArmA_Classes)
//      height       : the height the objects should be created at 
//      amount, delay: how many objects should be created, and at which time interval 
//      "random"     : flag that objects should be positioned randomly
// 
//  e.g.: nil=["paraiso", "BO_GBU12_LGB", 30, 50, 1,"random"] execVM "objectplacer.sqf" 
//           will create 50 bombs at a height of 30, at 1 second intervals, over a marker called "paraiso"
// 
// ---------------------------------------------------------------------------------------------------------
//
// By default the marker that's used to designate the area is hidden after the script initialization.
// To keep the marker visible, use "SHOWMARKER" in the parameter list.
//
// ---------------------------------------------------------------------------------------------------------
// 
// You can attach an init command to the objects created by adding [...,"init:","initstring"] to the parameter list
// e.g. nil=["area", "M113", 0, 20,0, "random", "init:","this setdammage 1"] execVM "objectplacer.sqf" 
//
// ---------------------------------------------------------------------------------------------------------
// 
//  If no parameters except for the marker name are given, a matrix of bombs will be created,
//  at 10 meter distances, and at a height of 30 (e.g. nil=["paraiso"] execVM "objectplacer.sqf")
// 
// =========================================================================================================

if (!isNull(server) && !(local server)) exitWith{};
// replace the above check with the following line for ArmA 1.06 and higher
// if !(isServer) exitWith {};


if (isNil("KRON_OP_Tot")) then {
	KRON_OP_Tot=0;
};

_getArg = {private["_cLC","_cUC","_arg","_list","_a","_v"]; _cLC=_this select 0; _cUC=_this select 1; _arg=_this select 2; _list=_this select 3; _a=-1; {_a=_a+1; _v=format["%1",_list select _a]; if ((_v==_cLC) || (_v==_cUC)) then {_arg=(_list select _a+1)}} foreach _list; _arg};

_areamarker = _this select 0;

_areadir = (markerDir _areamarker);

_xdist=10; _ydist=10; _object="BO_GBU12_LGB"; _z=200; 
if (count _this>1) then {_object = _this select 1};
if (_object=="bomb") then {_object="BO_GBU12_LGB"};
if (count _this>2) then {_z = _this select 2};
if (count _this>3) then {_xdist = _this select 3};
if (count _this>4) then {_ydist = _this select 4};
_initstr = ["init:","INIT:","",_this] call _getArg;

// size 
_areasize = getMarkerSize _areamarker;
_sizeX = _areasize select 0;
_sizeY = _areasize select 1;

// position 
_centerpos = getMarkerPos _areamarker;
_centerX = abs(_centerpos select 0);
_centerY = abs(_centerpos select 1);

// hide marker
if (!("SHOWMARKER" in _this) && !("showmarker" in _this)) then {
	_areamarker setMarkerPos [abs(getMarkerPos _areamarker select 0)*-1, abs(getMarkerPos _areamarker select 1)*-1];
};

// trig values
_cosdir=cos(_areadir*-1);
_sindir=sin(_areadir*-1);

_rotpoint = {private["_cx","_cy","_rx","_ry","_cd","_sd","_ad","_tx","_ty","_xout","_yout"];_cx=_this select 0; _cy=_this select 1; _rx=_this select 2; _ry=_this select 3; _cd=_this select 4; _sd=_this select 5; _ad=_this select 6; _tx=_this select 7; _ty=_this select 8; _xout=if (_ad!=0) then {_cx+ (_cd*_tx - _sd*_ty)} else {_cx+_tx}; _yout=if (_ad!=0) then {_cy+ (_sd*_tx + _cd*_ty)} else {_cy+_ty}; [_xout,_yout]};

_makeobj = {
	private["_o","_o1","_x","_y","_z","_d","_in"]; 
	_o=_this select 0; 
	_x=_this select 1;  
	_y=_this select 2;  
	_z=_this select 3;  
	_d=_this select 4; 
	_in=_this select 5; 
	_o1 = _o createVehicle [_x,_y,_z]; 
	_o1 setPos [_x,_y,_z]; 
	_o1 setDir _d; 
	if (_in!="") then {_o1 setVehicleInit _in};
	_o1;
};


if (("RANDOM" in _this) || ("random" in _this)) then {
	for [{_i=0}, {_i<=_xdist}, {_i=_i+1}] do {
		KRON_OP_Tot=KRON_OP_Tot+1;
		_x=random (_sizeX*2)-_sizeX; 
		_y=random (_sizeY*2)-_sizeY; 
		_pos = [_centerX,_centerY,_sizeX,_sizeY,_cosdir,_sindir,_areadir,_x,_y] call _rotpoint;
		_tx = _pos select 0; 
		_ty = _pos select 1; 
		_obj=[_object,_tx,_ty,_z,random 360,_initstr] call _makeobj;
		sleep _ydist;
	};
	
} else {

	if (_xdist==0) then {_xdist=10};
	if (_ydist==0) then {_ydist=10};

	KRON_OP_Tot=0;
	for [{_y=_sizeY*-1}, {_y<=_sizeY}, {_y=_y+_ydist}] do {
		for [{_x=_sizeX*-1}, {_x<=_sizeX}, {_x=_x+_xdist}] do {
			KRON_OP_Tot=KRON_OP_Tot+1;
			_pos = [_centerX,_centerY,_sizeX,_sizeY,_cosdir,_sindir,_areadir,_x,_y] call _rotpoint;
			_tx = _pos select 0; 
			_ty = _pos select 1; 
			//player sidechat format["%1,%2",_tx,_ty];
			_obj=[_object,_tx,_ty,_z,_areadir,_initstr] call _makeobj;
		};
		sleep .1;
	};
	//hint format["total objects: %1",KRON_OP_Tot]
};

processInitCommands;